package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/expression"
	"git.akyoto.dev/cli/q/src/build/types"
)

// CompileDefinition compiles a variable definition.
func (f *Function) CompileDefinition(node *ast.Define) error {
	left := node.Expression.Children[0]
	right := node.Expression.Children[1]

	if left.IsLeaf() {
		variable, err := f.Define(left)

		if err != nil {
			return err
		}

		typ, err := f.ExpressionToRegister(right, variable.Register)

		if err != nil {
			return err
		}

		variable.Type = typ

		if variable.Type == types.Invalid {
			return errors.New(errors.UnknownType, f.File, node.Expression.Token.End())
		}

		f.AddVariable(variable)
		return nil
	}

	if !ast.IsFunctionCall(right) {
		return errors.New(errors.NotImplemented, f.File, node.Expression.Token.Position)
	}

	count := 0
	_, err := f.CompileCall(right)

	if err != nil {
		return err
	}

	return left.EachLeaf(func(leaf *expression.Expression) error {
		variable, err := f.Define(leaf)

		if err != nil {
			return err
		}

		f.RegisterRegister(asm.MOVE, variable.Register, f.CPU.Output[count])
		f.AddVariable(variable)
		count++
		return nil
	})
}
